# Admone Zsh prompt
# Copyright Armaan Bhojwani 2020 <code@armaanb.net>
# MIT licensed, see the LICENSE file for more information.
# Git repo at <https://codeberg.org/armaan/admone>

#######################################################################
# USER CONFIGURATIONS
#######################################################################

PROMPT_CHAR="➤"
  # PROMPT_CHAR="❯❯❯"
  # PROMPT_CHAR="$"
  # PROMPT_CHAR="%"
  # PROMPT_CHAR=">"
  # PROMPT_CHAR="➢"
  # PROMPT_CHAR="➪"
PRE_UPPER="╭"
PRE_LOWER="╰"
  # PRE_UPPER="┌"
  # PRE_LOWER="└"
BLOCK_SEPARATOR="─"
  # BLOCK_SEPARATOR="∙"
BLOCK_LEFT="["
BLOCK_RIGHT="]"
  # BLOCK_LEFT="("
  # BLOCK_RIGHT=")"
SHOW_0_EXIT_CODE="0"
INCLUDE="admone-pwd;admone-vcs;admone-exit-code"
FORMAT_LOWER="%B%F{%(?.cyan.red)}"

TRANSIENT_PROMPT="0" # This is buggy
SHORT_PROMPT="➤"
FORMAT_SHORT="%B%F{%(?.cyan.red)}"

zstyle ':vcs_info:git:*' formats '%b%u%c'
zstyle ':vcs_info:git:*' actionformats '%b|%a%u%c'
zstyle ':vcs_info:*' unstagedstr ' *'
zstyle ':vcs_info:*' stagedstr ' +'

#######################################################################
# Functions
#######################################################################

admone-pwd() {
  echo "%F{blue}%~%f"
}
admone-vcs() {
  echo "%F{green}${vcs_info_msg_0_}%f"
}
admone-exit-code() {
  ([[ $SHOW_0_EXIT_CODE == "1" ]] && echo "%(?,%F{cyan}%?%f,%F{red}%?%f)") ||
    echo "%(?,,%F{red}%?%f)"
}
admone-date() {
  echo $(date -I)
}
admone-12hr() {
  echo $(date +%r)
}
admone-24hr() {
  echo $(date +%T)
}
# Intergration with https://codeberg.org/armaan/zsh-command-time
admone-time() {
  echo "%F{cyan}$timer_show%f"
}

#######################################################################
# BACKEND
#######################################################################

# Cursor shape depends on vi mode
function zle-keymap-select zle-line-init zle-line-finish {
  case $KEYMAP in
    vicmd)
      # Block
      print -n -- "\E]50;CursorShape=0\C-G"
      ;;
    viins|main)
      # Beam
      print -n -- "\E]50;CursorShape=1\C-G"
      ;;
  esac
  zle reset-prompt
  zle -R
}

set-prompt() {
  # Reset prompts
  upper="${PRE_UPPER}"
  lower="${PRE_LOWER}${FORMAT_LOWER}${PROMPT_CHAR}%f%b "

  # Add in functions as defined above
  NUM_FUNCS="$(echo $INCLUDE | grep -o ';' | wc -l)"

  THE_COMMAND="$(echo $INCLUDE | cut -d ';' -f 1)"
  upper+="$BLOCK_LEFT$($THE_COMMAND)$BLOCK_RIGHT"

  for (( i = 0; i < $NUM_FUNCS; i++ )); do
    THE_COMMAND="$(echo $INCLUDE | cut -d ';' -f $(($i + 2)))"
    upper+="$BLOCK_SEPARATOR$BLOCK_LEFT$($THE_COMMAND)$BLOCK_RIGHT"
  done

  # Add line above
  [[ -v PS1_NL ]] && echo || PS1_NL=

  PROMPT=$upper$'\n'$lower
}


# Set everything!
if [[ $TRANSIENT_PROMPT == "1" ]]; then
  SHORT_PROMPT=$FORMAT_SHORT$SHORT_PROMPT' '%f%b

  set-transient-prompt() {
    if [[ $PROMPT != $SHORT_PROMPT ]]; then
      PROMPT=$SHORT_PROMPT
      zle .reset-prompt
    fi
  }

  zle-line-finish() {
    set-transient-prompt
  }

  trap 'set-transient-prompt; return 130' INT
fi

if [[ ! $(echo $INCLUDE | grep -q "vcs") ]]; then
  zstyle ':vcs_info:*' check-for-changes true
  autoload -Uz add-zsh-hook vcs-info
  add-zsh-hook precmd vcs_info
fi

autoload -Uz add-zsh-hook
add-zsh-hook precmd set-prompt

zle -N zle-line-init
zle -N zle-line-finish
zle -N zle-keymap-select
